// LSystemDoc.cpp : implementation of the CLSystemDoc class
//

#include "stdafx.h"
#include "LSystem.h"
#include <fstream.h>

#include "LSystemDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#include "math.h"

#define M_PI        3.14159265358979323846
#define STACK_SIZE  512
#define PUSH(X) stack[sp++] = X
#define POP(X) X = stack[--sp]

/////////////////////////////////////////////////////////////////////////////
// CLSystemDoc

IMPLEMENT_DYNCREATE(CLSystemDoc, CDocument)

BEGIN_MESSAGE_MAP(CLSystemDoc, CDocument)
	//{{AFX_MSG_MAP(CLSystemDoc)
	ON_COMMAND(ID_PARAMS, OnParams)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLSystemDoc construction/destruction

CLSystemDoc::CLSystemDoc()
{
	// TODO: add one-time construction code here
	beg = end = NULL;
	sp = 0;
}

CLSystemDoc::~CLSystemDoc()
{
}

BOOL CLSystemDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	dlg.m_Axiom = _T("F");
	dlg.m_NewF = _T("F");
	dlg.m_NewX = _T("");
	dlg.m_NewY = _T("");
	dlg.m_Newb = _T("");
	dlg.m_AngleStep = 1;
	dlg.m_StartAngle = 0;
	dlg.m_Iterations = 0;

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CLSystemDoc serialization

void CLSystemDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
		ar << dlg.m_Axiom;
		ar << dlg.m_NewF;
		ar << dlg.m_NewX;
		ar << dlg.m_NewY;
		ar << dlg.m_Newb;
		ar << dlg.m_AngleStep;
		ar << dlg.m_StartAngle;
		ar << dlg.m_Iterations;
	}
	else
	{
		// TODO: add loading code here
		ar >> dlg.m_Axiom;
		ar >> dlg.m_NewF;
		ar >> dlg.m_NewX;
		ar >> dlg.m_NewY;
		ar >> dlg.m_Newb;
		ar >> dlg.m_AngleStep;
		ar >> dlg.m_StartAngle;
		ar >> dlg.m_Iterations;
		Calculate();
	}
}

/////////////////////////////////////////////////////////////////////////////
// CLSystemDoc diagnostics

#ifdef _DEBUG
void CLSystemDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CLSystemDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLSystemDoc commands

void CLSystemDoc::OnParams() 
{
	// TODO: Add your command handler code here
	
	CString s1 = dlg.m_Axiom;
	CString s2 = dlg.m_NewF;
	CString s3 = dlg.m_NewX;
	CString s4 = dlg.m_NewY;
	CString s5 = dlg.m_Newb;
	int i1 = dlg.m_AngleStep;
	int i2 = dlg.m_StartAngle;
	int i3 = dlg.m_Iterations;
	if (dlg.DoModal() != IDOK) return;
	if (s1 == dlg.m_Axiom &&
		s2 == dlg.m_NewF &&
		s3 == dlg.m_NewX &&
		s4 == dlg.m_NewY &&
		s5 == dlg.m_Newb &&
		i1 == dlg.m_AngleStep &&
		i2 == dlg.m_StartAngle &&
		i3 == dlg.m_Iterations)
		return;
	Calculate();
	SetModifiedFlag();
	UpdateAllViews(NULL);	
}

void CLSystemDoc::Calculate()
{
	x_min = 1.7E+308, y_min = 1.7E+308;
	x_max = -1.7E+308, y_max = -1.7E+308;
	double x0 = 0.0, y0 = 0.0, x = 0.0, y = 0.0;
	int i, n;
	unsigned m;
	double beg_alpha;
	double alpha = M_PI / dlg.m_AngleStep;
	int level = dlg.m_Iterations;

	stack = new double[STACK_SIZE];
	char *W = new char[strlen(dlg.m_Axiom) + 1], *T;
	strcpy(W, dlg.m_Axiom);

	m = strlen(dlg.m_NewF);
	if (strlen(dlg.m_NewX) > m) m = strlen(dlg.m_NewX);
	if (strlen(dlg.m_NewY) > m) m = strlen(dlg.m_NewY);
	if (strlen(dlg.m_Newb) > m) m = strlen(dlg.m_Newb);
	if (!dlg.m_StartAngle) 
		beg_alpha = 0.0;
	else
		beg_alpha = M_PI / dlg.m_StartAngle;

	while (level)
	{
		n = strlen(W);
		T = new char[n * m + 1];
		T[0] = 0;
		for (i = 0; i < n; i++)
			switch (W[i])
			{
				case '+': strcat(T, "+"); break;
				case '-': strcat(T, "-"); break;
				case '[': strcat(T, "["); break;
				case ']': strcat(T, "]"); break;
				case 'F': strcat(T, dlg.m_NewF); break;
				case 'X': strcat(T, dlg.m_NewX); break;
				case 'Y': strcat(T, dlg.m_NewY); break;
				case 'b': strcat(T, dlg.m_Newb);
			}
		delete W;
		W = new char[strlen(T) + 1];
		strcpy(W, T);
		delete T;
		level--;
	}

	DeleteList();
	double a = beg_alpha;
	n = strlen(W);
	for (i = 0; i < n; i++)
	{
		switch(W[i])
		{
			case '+': a += alpha; break;
			case '-': a -= alpha; break;
			case 'F': 
				x = x0 + cos(a);
                y = y0 + sin(a);
                AppendList(x0, y0, x, y);
				SetMaxMin(x0, y0);
				SetMaxMin(x, y);
                x0 = x;
                y0 = y;
                break;
			case 'b': 
				x0 += cos(a);
                y0 += sin(a);
                break;
			case '[': Push(x0, y0, a); break;
			case ']': Pop(x0, y0, (double&)a);
		}
	}
	delete W;
	delete stack;
}

void CLSystemDoc::Push(double x, double y, double a)
{
	PUSH(x);
	PUSH(y);
	PUSH(a);
}

void CLSystemDoc::Pop(double& x, double& y, double& a)
{
	POP(a);
	POP(y);
	POP(x);
}

void CLSystemDoc::AppendList(double x1, double y1, double x2, double y2)
{
	XYList *el = new XYList;

	el -> x1 = x1;
	el -> y1 = y1;
	el -> x2 = x2;
	el -> y2 = y2;
	el -> next = NULL;

	if (beg == NULL)
		beg = el;
	else
		end -> next = el;

	end = el;
}

void CLSystemDoc::DeleteList()
{
	XYList *el = beg;

	while (el)
	{
		beg = beg -> next;
		delete el;
		el = beg;
	}

	beg = end = NULL;
}

void CLSystemDoc::SetMaxMin(double x, double y)
{
	if (x < x_min) x_min = x;
	if (y < y_min) y_min = y;
	if (x > x_max) x_max = x;
	if (y > y_max) y_max = y;
}

void CLSystemDoc::DeleteContents() 
{
	// TODO: Add your specialized code here and/or call the base class
	DeleteList();
	CDocument::DeleteContents();
}
